import React from 'react';
import { EditorStack } from '../EditorStack.js';

const EditorRows = ({ children }) => {
  return /* @__PURE__ */ React.createElement(EditorStack, { gap: 0.5, direction: "column" }, children);
};

export { EditorRows };
//# sourceMappingURL=EditorRows.js.map
