import { scheduled } from '../scheduled/scheduled.js';
import { innerFrom } from './innerFrom.js';

function from(input, scheduler) {
    return scheduler ? scheduled(input, scheduler) : innerFrom(input);
}

export { from };
//# sourceMappingURL=from.js.map
