import React from 'react';

const QueryEditorRow = (props) => {
  var _a, _b;
  const className = (_a = props.className) != null ? _a : "width-8";
  const noFillEnd = (_b = props.noFillEnd) != null ? _b : false;
  return /* @__PURE__ */ React.createElement("div", { className: "gf-form" }, props.label && /* @__PURE__ */ React.createElement("label", { className: `gf-form-label query-keyword ${className}` }, props.label), props.children, /* @__PURE__ */ React.createElement("div", { className: "gf-form--grow" }, noFillEnd || /* @__PURE__ */ React.createElement("div", { className: "gf-form-label gf-form-label--grow" })));
};

export { QueryEditorRow };
//# sourceMappingURL=QueryEditorRow.js.map
