"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3863],{33863(_,n,e){e.d(n,{MergeTransformerEditor:()=>f});var r=e(14553),a=e(52358),s=e(61191);const f=({input:t,options:o,onChange:g})=>t.length<=1?(0,r.jsx)(s.P,{children:(0,r.jsx)(a.x6,{i18nKey:"transformers.merge-transformer-editor.merge-effect-applied-single-frame",children:"Merge has no effect when applied on a single frame."})}):null}}]);

//# sourceMappingURL=3863-react19.2c8fb0795ecbe4fea744.js.map