import { innerFrom } from '../observable/innerFrom.js';
import { observeOn } from '../operators/observeOn.js';
import { subscribeOn } from '../operators/subscribeOn.js';

function scheduleObservable(input, scheduler) {
    return innerFrom(input).pipe(subscribeOn(scheduler), observeOn(scheduler));
}

export { scheduleObservable };
//# sourceMappingURL=scheduleObservable.js.map
