$item-offset-left: 10px !default;
$item-offset: 5px !default;
$rulegroup-offset-left: 10px !default;
$rulegroup-offset: 5px !default;
$rule-padding: 5px !default;
$rule-parts-distance: 5px !default;
$drag-offset-right: 5px !default;
$rule-group-actions-offset-left: 5px !default;


@import './styles.scss';

.query-builder {
  .group--conjunctions {
    margin-left: -1px !important;
    margin-top: -1px !important;
  }
}
