// is a value between two other values

export default (
    lowerBound: number,
    upperBound: number,
  ): ((a: number) => boolean) =>
  (value: number): boolean =>
    lowerBound <= value && value <= upperBound;
