import { isFunction } from './isFunction.js';

function isPromise(value) {
    return isFunction(value === null || value === undefined ? undefined : value.then);
}

export { isPromise };
//# sourceMappingURL=isPromise.js.map
