"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2182],{88552(s,e,a){s.exports=a.p+"static/img/provisioning.d25529e1.svg"}}]);

//# sourceMappingURL=RepositoryListPage.2801002897aa4785e591.js.map