import { __require as requireParse } from './parse.js';

var valid_1;
var hasRequiredValid;

function requireValid () {
	if (hasRequiredValid) return valid_1;
	hasRequiredValid = 1;

	const parse = requireParse();
	const valid = (version, options) => {
	  const v = parse(version, options);
	  return v ? v.version : null
	};
	valid_1 = valid;
	return valid_1;
}

export { requireValid as __require };
//# sourceMappingURL=valid.js.map
