"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[8638,9136],{26506(_,s,a){a.r(s),a.d(s,{default:()=>p});var e=a(74848),t=a(58348);function r(d){return(0,e.jsx)(t.default,{...d})}const p=r},97484(_,s,a){_.exports=a.p+"static/img/dashboard_library_1.93b7e752.jpg"},7728(_,s,a){_.exports=a.p+"static/img/dashboard_library_2.e540bb1d.jpg"},52621(_,s,a){_.exports=a.p+"static/img/dashboard_library_3.8f41d293.jpg"},36422(_,s,a){_.exports=a.p+"static/img/dashboard_library_4.0a1e811b.jpg"},26531(_,s,a){_.exports=a.p+"static/img/dashboard_library_5.7f4bc831.jpg"},96820(_,s,a){_.exports=a.p+"static/img/dashboard_library_6.cf5e2209.jpg"},12326(_,s,a){_.exports=a.p+"static/img/grafana_text_logo_dark.70a1e5ee.svg"},71542(_,s,a){_.exports=a.p+"static/img/grafana_text_logo_light.467e047c.svg"}}]);

//# sourceMappingURL=DashboardPageProxy.b34bbe9a041fff33cb16.js.map