"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3640],{73640(E,o,a){a.d(o,{TransposeTransformerEditor:()=>_});var e=a(14553),r=a(52358),t=a(72480),n=a(6634),f=a(87748),d=a(16244),m=a(75251);const _=({options:l,onChange:i})=>{const u=s=>{i({...l,emptyValue:s?.value})};return(0,e.jsxs)(e.Fragment,{children:[(0,e.jsx)(n.C,{children:(0,e.jsx)(t.I,{label:(0,r.t)("transformers.transpose-transfomer-editor.label-first-field-name","First field name"),labelWidth:24,children:(0,e.jsx)(d.p,{placeholder:(0,r.t)("transformers.transpose-transfomer-editor.placeholder-field","Field"),value:l.firstFieldName,onChange:s=>i({...l,firstFieldName:s.currentTarget.value}),width:25})})}),(0,e.jsx)(n.C,{children:(0,e.jsx)(t.I,{label:(0,r.t)("transformers.transpose-transfomer-editor.label-remaining-fields-name","Remaining fields name"),tooltip:(0,r.t)("transformers.transpose-transfomer-editor.tooltip-name-for-value-fields","Name for value fields"),labelWidth:24,children:(0,e.jsx)(d.p,{placeholder:(0,r.t)("transformers.transpose-transfomer-editor.placeholder-value","Value"),value:l.restFieldsName,onChange:s=>i({...l,restFieldsName:s.currentTarget.value}),width:25})})}),(0,e.jsx)(n.C,{children:(0,e.jsx)(t.I,{label:(0,r.t)("transformers.grouping-to-matrix-transformer-editor.label-empty-value","Empty value"),children:(0,e.jsx)(f.l6,{options:(0,m.wS)(),value:l.emptyValue,onChange:u,isClearable:!0})})})]})}}}]);

//# sourceMappingURL=3640-react19.3a3031d668e180413e69.js.map