"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3005],{93005(c,p,o){o.d(p,{PostHogBackend:()=>u});var n=o(6188),e=o(69853),s=o(57647);const d="https://us.i.posthog.com";class u{constructor(t){this.options=t,this.supportedEvents=[e.on.Pageview,e.on.Interaction,e.on.ExperimentView,e.on.Performance,e.on.MetaAnalytics,e.on.GrafanaJavascriptAgent],this.addEvent=a=>{if(window.posthog){if((0,n.nB)(a)){window.posthog.capture?.("$pageview",{path:a.payload.page});return}if((0,n.fX)(a)){window.posthog.capture?.(a.payload.interactionName,a.payload.properties);return}if((0,n.p6)(a)){window.posthog.capture?.("experiment_viewed",{experiment_id:a.payload.experimentId,experiment_group:a.payload.experimentGroup,experiment_variant:a.payload.experimentVariant});return}if(a.type===e.on.MetaAnalytics){window.posthog.capture?.(a.payload.eventName,a.payload);return}if(a.type===e.on.Performance){window.posthog.capture?.("performance_metric",{metric_name:a.payload.name,metric_value:a.payload.value});return}if(a.type===e.on.GrafanaJavascriptAgent){window.posthog.capture?.("grafana_javascript_agent_event",a.payload);return}}},this.flush=()=>{};const r=t.postHogHost||d;if(!(window.posthog&&window.posthog.__loaded)){const a=window.posthog=[];for(const i of["init","identify","capture"])a[i]=function(...g){a.push([i,...g])};(0,s.k)(`${r}/static/array.js`,!0)}window.posthog?.init?.(t.postHogToken,{api_host:r}),t.user&&window.posthog?.identify?.(t.user.analytics.identifier,{email:t.user.email,name:t.user.name,orgId:t.user.orgId,orgName:t.user.orgName,orgRole:t.user.orgRole})}}}}]);

//# sourceMappingURL=3005-react19.58555623616dfd0818e2.js.map